@@ -1,11 +1,9 @@
 /***************************************
- $Header: /home/amb/routino/src/RCS/nodes.c,v 1.39 2010/07/08 17:54:54 amb Exp $
-
  Node data type functions.
 
  Part of the Routino routing software.
  ******************/ /******************
- This file Copyright 2008-2010 Andrew M. Bishop
+ This file Copyright 2008-2011 Andrew M. Bishop
 
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU Affero General Public License as published by
@@ -26,50 +24,70 @@
 #include <stdlib.h>
 #include <math.h>
 
-#include "profiles.h"
 #include "nodes.h"
 #include "segments.h"
 #include "ways.h"
-#include "functions.h"
+
+#include "files.h"
+#include "profiles.h"
 
 
 /*++++++++++++++++++++++++++++++++++++++
   Load in a node list from a file.
 
-  Nodes* LoadNodeList Returns the node list.
+  Nodes *LoadNodeList Returns the node list.
 
   const char *filename The name of the file to load.
   ++++++++++++++++++++++++++++++++++++++*/
 
 Nodes *LoadNodeList(const char *filename)
 {
- void *data;
  Nodes *nodes;
+#if SLIM
+ int i;
+#endif
 
  nodes=(Nodes*)malloc(sizeof(Nodes));
 
- data=MapFile(filename);
+#if !SLIM
 
- /* Copy the Nodes structure from the loaded data */
+ nodes->data=MapFile(filename);
 
- *nodes=*((Nodes*)data);
+ /* Copy the NodesFile header structure from the loaded data */
 
- /* Adjust the pointers in the Nodes structure. */
+ nodes->file=*((NodesFile*)nodes->data);
 
- nodes->data=data;
- nodes->offsets=(index_t*)(data+sizeof(Nodes));
- nodes->nodes=(Node*)(data+(sizeof(Nodes)+(nodes->latbins*nodes->lonbins+1)*sizeof(index_t)));
+ /* Set the pointers in the Nodes structure. */
+
+ nodes->offsets=(index_t*)(nodes->data+sizeof(NodesFile));
+ nodes->nodes  =(Node*   )(nodes->data+sizeof(NodesFile)+(nodes->file.latbins*nodes->file.lonbins+1)*sizeof(index_t));
+
+#else
+
+ nodes->fd=ReOpenFile(filename);
+
+ /* Copy the NodesFile header structure from the loaded data */
+
+ ReadFile(nodes->fd,&nodes->file,sizeof(NodesFile));
+
+ nodes->nodesoffset=sizeof(NodesFile)+(nodes->file.latbins*nodes->file.lonbins+1)*sizeof(index_t);
+
+ for(i=0;i<sizeof(nodes->cached)/sizeof(nodes->cached[0]);i++)
+    nodes->incache[i]=NO_NODE;
+
+#endif
 
  return(nodes);
 }
 
 
 /*++++++++++++++++++++++++++++++++++++++
-  Find the closest node given its latitude, longitude and optionally profile.
+  Find the closest node given its latitude, longitude and optionally the profile
+  of the mode of transport that must be able to move to/from this node.
 
   index_t FindClosestNode Returns the closest node.
 
-  Nodes* nodes The set of nodes to search.
+  Nodes *nodes The set of nodes to search.
 
   Segments *segments The set of segments to use.
 
@@ -79,20 +97,21 @@ Nodes *LoadNodeList(const char *filename)
 
   double longitude The longitude to look for.
 
-  distance_t distance The maximum distance to look.
+  distance_t distance The maximum distance to look from the specified coordinates.
 
   Profile *profile The profile of the mode of transport (or NULL).
 
   distance_t *bestdist Returns the distance to the best node.
   ++++++++++++++++++++++++++++++++++++++*/
 
-index_t FindClosestNode(Nodes* nodes,Segments *segments,Ways *ways,double latitude,double longitude,
+index_t FindClosestNode(Nodes *nodes,Segments *segments,Ways *ways,double latitude,double longitude,
                         distance_t distance,Profile *profile,distance_t *bestdist)
 {
- ll_bin_t   latbin=latlong_to_bin(radians_to_latlong(latitude ))-nodes->latzero;
- ll_bin_t   lonbin=latlong_to_bin(radians_to_latlong(longitude))-nodes->lonzero;
+ ll_bin_t   latbin=latlong_to_bin(radians_to_latlong(latitude ))-nodes->file.latzero;
+ ll_bin_t   lonbin=latlong_to_bin(radians_to_latlong(longitude))-nodes->file.lonzero;
  int        delta=0,count;
- index_t    i,bestn=NO_NODE;
+ index_t    i,index1,index2;
+ index_t    bestn=NO_NODE;
  distance_t bestd=INF_DISTANCE;
 
  /* Start with the bin containing the location, then spiral outwards. */
@@ -102,30 +121,30 @@ index_t FindClosestNode(Nodes* nodes,Segments *segments,Ways *ways,double latitu
     int latb,lonb,llbin;
 
     count=0;
-   
+
     for(latb=latbin-delta;latb<=latbin+delta;latb++)
       {
-       if(latb<0 || latb>=nodes->latbins)
+       if(latb<0 || latb>=nodes->file.latbins)
           continue;
 
        for(lonb=lonbin-delta;lonb<=lonbin+delta;lonb++)
          {
-          if(lonb<0 || lonb>=nodes->lonbins)
+          if(lonb<0 || lonb>=nodes->file.lonbins)
              continue;
 
           if(abs(latb-latbin)<delta && abs(lonb-lonbin)<delta)
              continue;
 
-          llbin=lonb*nodes->latbins+latb;
+          llbin=lonb*nodes->file.latbins+latb;
 
           /* Check if this grid square has any hope of being close enough */
 
           if(delta>0)
             {
-             double lat1=latlong_to_radians(bin_to_latlong(nodes->latzero+latb));
-             double lon1=latlong_to_radians(bin_to_latlong(nodes->lonzero+lonb));
-             double lat2=latlong_to_radians(bin_to_latlong(nodes->latzero+latb+1));
-             double lon2=latlong_to_radians(bin_to_latlong(nodes->lonzero+lonb+1));
+             double lat1=latlong_to_radians(bin_to_latlong(nodes->file.latzero+latb));
+             double lon1=latlong_to_radians(bin_to_latlong(nodes->file.lonzero+lonb));
+             double lat2=latlong_to_radians(bin_to_latlong(nodes->file.latzero+latb+1));
+             double lon2=latlong_to_radians(bin_to_latlong(nodes->file.lonzero+lonb+1));
 
              if(latb==latbin)
                {
@@ -157,10 +176,14 @@ index_t FindClosestNode(Nodes* nodes,Segments *segments,Ways *ways,double latitu
 
           /* Check every node in this grid square. */
 
-          for(i=nodes->offsets[llbin];i<nodes->offsets[llbin+1];i++)
+          index1=LookupNodeOffset(nodes,llbin);
+          index2=LookupNodeOffset(nodes,llbin+1);
+
+          for(i=index1;i<index2;i++)
             {
-             double lat=latlong_to_radians(bin_to_latlong(nodes->latzero+latb)+off_to_latlong(nodes->nodes[i].latoffset));
-             double lon=latlong_to_radians(bin_to_latlong(nodes->lonzero+lonb)+off_to_latlong(nodes->nodes[i].lonoffset));
+             Node *node=LookupNode(nodes,i,1);
+             double lat=latlong_to_radians(bin_to_latlong(nodes->file.latzero+latb)+off_to_latlong(node->latoffset));
+             double lon=latlong_to_radians(bin_to_latlong(nodes->file.lonzero+lonb)+off_to_latlong(node->lonoffset));
 
              distance_t dist=Distance(lat,lon,latitude,longitude);
 
@@ -172,11 +195,11 @@ index_t FindClosestNode(Nodes* nodes,Segments *segments,Ways *ways,double latitu
 
                    /* Decide if this is node is valid for the profile */
 
-                   segment=FirstSegment(segments,nodes,i);
+                   segment=FirstSegment(segments,nodes,i,1);
 
                    do
                      {
-                      Way *way=LookupWay(ways,segment->way);
+                      Way *way=LookupWay(ways,segment->way,1);
 
                       if(way->allow&profile->allow)
                          break;
@@ -208,13 +231,15 @@ index_t FindClosestNode(Nodes* nodes,Segments *segments,Ways *ways,double latitu
 
 
 /*++++++++++++++++++++++++++++++++++++++
-  Find the closest segment to a latitude, longitude and optionally profile.
+  Find the closest point on the closest segment given its latitude, longitude
+  and optionally the profile of the mode of transport that must be able to move
+  along this segment.
 
-  Segment *FindClosestSegment Returns the closest segment.
+  index_t FindClosestSegment Returns the closest segment index.
 
-  Nodes* nodes The set of nodes to search.
+  Nodes *nodes The set of nodes to use.
 
-  Segments *segments The set of segments to use.
+  Segments *segments The set of segments to search.
 
   Ways *ways The set of ways to use.
 
@@ -222,31 +247,32 @@ index_t FindClosestNode(Nodes* nodes,Segments *segments,Ways *ways,double latitu
 
   double longitude The longitude to look for.
 
-  distance_t distance The maximum distance to look.
+  distance_t distance The maximum distance to look from the specified coordinates.
 
   Profile *profile The profile of the mode of transport (or NULL).
 
-  distance_t *bestdist Returns the distance to the best segment.
+  distance_t *bestdist Returns the distance to the closest point on the best segment.
 
-  index_t *bestnode1 Returns the best node at one end.
+  index_t *bestnode1 Returns the index of the node at one end of the closest segment.
 
-  index_t *bestnode2 Returns the best node at the other end.
+  index_t *bestnode2 Returns the index of the node at the other end of the closest segment.
 
-  distance_t *bestdist1 Returns the distance to the best node at one end.
+  distance_t *bestdist1 Returns the distance along the segment to the node at one end.
 
-  distance_t *bestdist2 Returns the distance to the best node at the other end.
+  distance_t *bestdist2 Returns the distance along the segment to the node at the other end.
   ++++++++++++++++++++++++++++++++++++++*/
 
-Segment *FindClosestSegment(Nodes* nodes,Segments *segments,Ways *ways,double latitude,double longitude,
-                            distance_t distance,Profile *profile, distance_t *bestdist,
-                            index_t *bestnode1,index_t *bestnode2,distance_t *bestdist1,distance_t *bestdist2)
+index_t FindClosestSegment(Nodes *nodes,Segments *segments,Ways *ways,double latitude,double longitude,
+                           distance_t distance,Profile *profile, distance_t *bestdist,
+                           index_t *bestnode1,index_t *bestnode2,distance_t *bestdist1,distance_t *bestdist2)
 {
- ll_bin_t   latbin=latlong_to_bin(radians_to_latlong(latitude ))-nodes->latzero;
- ll_bin_t   lonbin=latlong_to_bin(radians_to_latlong(longitude))-nodes->lonzero;
+ ll_bin_t   latbin=latlong_to_bin(radians_to_latlong(latitude ))-nodes->file.latzero;
+ ll_bin_t   lonbin=latlong_to_bin(radians_to_latlong(longitude))-nodes->file.lonzero;
  int        delta=0,count;
- index_t    i,bestn1=NO_NODE,bestn2=NO_NODE;
+ index_t    i,index1,index2;
+ index_t    bestn1=NO_NODE,bestn2=NO_NODE;
  distance_t bestd=INF_DISTANCE,bestd1=INF_DISTANCE,bestd2=INF_DISTANCE;
- Segment   *bests=NULL;
+ index_t    bests=NO_SEGMENT;
 
  /* Start with the bin containing the location, then spiral outwards. */
 
@@ -255,30 +281,30 @@ Segment *FindClosestSegment(Nodes* nodes,Segments *segments,Ways *ways,double la
     int latb,lonb,llbin;
 
     count=0;
-   
+
     for(latb=latbin-delta;latb<=latbin+delta;latb++)
       {
-       if(latb<0 || latb>=nodes->latbins)
+       if(latb<0 || latb>=nodes->file.latbins)
           continue;
 
        for(lonb=lonbin-delta;lonb<=lonbin+delta;lonb++)
          {
-          if(lonb<0 || lonb>=nodes->lonbins)
+          if(lonb<0 || lonb>=nodes->file.lonbins)
              continue;
 
           if(abs(latb-latbin)<delta && abs(lonb-lonbin)<delta)
              continue;
 
-          llbin=lonb*nodes->latbins+latb;
+          llbin=lonb*nodes->file.latbins+latb;
 
           /* Check if this grid square has any hope of being close enough */
 
           if(delta>0)
             {
-             double lat1=latlong_to_radians(bin_to_latlong(nodes->latzero+latb));
-             double lon1=latlong_to_radians(bin_to_latlong(nodes->lonzero+lonb));
-             double lat2=latlong_to_radians(bin_to_latlong(nodes->latzero+latb+1));
-             double lon2=latlong_to_radians(bin_to_latlong(nodes->lonzero+lonb+1));
+             double lat1=latlong_to_radians(bin_to_latlong(nodes->file.latzero+latb));
+             double lon1=latlong_to_radians(bin_to_latlong(nodes->file.lonzero+lonb));
+             double lat2=latlong_to_radians(bin_to_latlong(nodes->file.latzero+latb+1));
+             double lon2=latlong_to_radians(bin_to_latlong(nodes->file.lonzero+lonb+1));
 
              if(latb==latbin)
                {
@@ -310,10 +336,14 @@ Segment *FindClosestSegment(Nodes* nodes,Segments *segments,Ways *ways,double la
 
           /* Check every node in this grid square. */
 
-          for(i=nodes->offsets[llbin];i<nodes->offsets[llbin+1];i++)
+          index1=LookupNodeOffset(nodes,llbin);
+          index2=LookupNodeOffset(nodes,llbin+1);
+
+          for(i=index1;i<index2;i++)
             {
-             double lat1=latlong_to_radians(bin_to_latlong(nodes->latzero+latb)+off_to_latlong(nodes->nodes[i].latoffset));
-             double lon1=latlong_to_radians(bin_to_latlong(nodes->lonzero+lonb)+off_to_latlong(nodes->nodes[i].lonoffset));
+             Node *node=LookupNode(nodes,i,1);
+             double lat1=latlong_to_radians(bin_to_latlong(nodes->file.latzero+latb)+off_to_latlong(node->latoffset));
+             double lon1=latlong_to_radians(bin_to_latlong(nodes->file.lonzero+lonb)+off_to_latlong(node->lonoffset));
              distance_t dist1;
 
              dist1=Distance(lat1,lon1,latitude,longitude);
@@ -324,7 +354,7 @@ Segment *FindClosestSegment(Nodes* nodes,Segments *segments,Ways *ways,double la
 
                 /* Check each segment for closeness and if valid for the profile */
 
-                segment=FirstSegment(segments,nodes,i);
+                segment=FirstSegment(segments,nodes,i,1);
 
                 do
                   {
@@ -333,7 +363,7 @@ Segment *FindClosestSegment(Nodes* nodes,Segments *segments,Ways *ways,double la
                       Way *way=NULL;
 
                       if(profile)
-                         way=LookupWay(ways,segment->way);
+                         way=LookupWay(ways,segment->way,1);
 
                       if(!profile || way->allow&profile->allow)
                         {
@@ -351,7 +381,11 @@ Segment *FindClosestSegment(Nodes* nodes,Segments *segments,Ways *ways,double la
                          dist3a=((double)dist1*(double)dist1-(double)dist2*(double)dist2+(double)dist3*(double)dist3)/(2.0*(double)dist3);
                          dist3b=(double)dist3-dist3a;
 
-                         if(dist3a>=0 && dist3b>=0)
+                         if((dist1+dist2)<dist3)
+                           {
+                            distp=0;
+                           }
+                         else if(dist3a>=0 && dist3b>=0)
                             distp=sqrt((double)dist1*(double)dist1-dist3a*dist3a);
                          else if(dist3a>0)
                            {
@@ -368,11 +402,23 @@ Segment *FindClosestSegment(Nodes* nodes,Segments *segments,Ways *ways,double la
 
                          if(distp<(double)bestd)
                            {
-                            bests=segment;
-                            bestn1=i;
-                            bestn2=OtherNode(segment,i);
-                            bestd1=(distance_t)dist3a;
-                            bestd2=(distance_t)dist3b;
+                            bests=IndexSegment(segments,segment);
+
+                            if(segment->node1==i)
+                              {
+                               bestn1=i;
+                               bestn2=OtherNode(segment,i);
+                               bestd1=(distance_t)dist3a;
+                               bestd2=(distance_t)dist3b;
+                              }
+                            else
+                              {
+                               bestn1=OtherNode(segment,i);
+                               bestn2=i;
+                               bestd1=(distance_t)dist3b;
+                               bestd2=(distance_t)dist3a;
+                              }
+
                             bestd=(distance_t)distp;
                            }
                         }
@@ -407,7 +453,7 @@ Segment *FindClosestSegment(Nodes* nodes,Segments *segments,Ways *ways,double la
 /*++++++++++++++++++++++++++++++++++++++
   Get the latitude and longitude associated with a node.
 
-  Nodes *nodes The set of nodes.
+  Nodes *nodes The set of nodes to use.
 
   index_t index The node index.
 
@@ -418,16 +464,17 @@ Segment *FindClosestSegment(Nodes* nodes,Segments *segments,Ways *ways,double la
 
 void GetLatLong(Nodes *nodes,index_t index,double *latitude,double *longitude)
 {
- Node *node=&nodes->nodes[index];
+ Node *node=LookupNode(nodes,index,2);
  int latbin=-1,lonbin=-1;
  int start,end,mid;
+ index_t offset;
 
- /* Binary search - search key closest below is required.
+ /* Binary search - search key exact match only is required.
   *
   *  # <- start  |  Check mid and move start or end if it doesn't match
   *  #           |
-  *  #           |  Since an inexact match is wanted we must set end=mid-1
-  *  # <- mid    |  or start=mid because we know that mid doesn't match.
+  *  #           |  Since an exact match is wanted we can set end=mid-1
+  *  # <- mid    |  or start=mid+1 because we know that mid doesn't match.
   *  #           |
   *  #           |  Eventually either end=start or end=start+1 and one of
   *  # <- end    |  start or end is the wanted one.
@@ -436,63 +483,73 @@ void GetLatLong(Nodes *nodes,index_t index,double *latitude,double *longitude)
  /* Search for longitude */
 
  start=0;
- end=nodes->lonbins-1;
+ end=nodes->file.lonbins-1;
 
  do
    {
-    mid=(start+end)/2;                                 /* Choose mid point */
+    mid=(start+end)/2;                  /* Choose mid point */
 
-    if(nodes->offsets[nodes->latbins*mid]<index)       /* Mid point is too low */
+    offset=LookupNodeOffset(nodes,nodes->file.latbins*mid);
+
+    if(offset<index)                    /* Mid point is too low */
        start=mid;
-    else if(nodes->offsets[nodes->latbins*mid]>index)  /* Mid point is too high */
+    else if(offset>index)               /* Mid point is too high */
        end=mid-1;
-    else                                               /* Mid point is correct */
+    else                                /* Mid point is correct */
       {lonbin=mid;break;}
    }
  while((end-start)>1);
 
  if(lonbin==-1)
    {
-    if(nodes->offsets[nodes->latbins*end]>index)
+    offset=LookupNodeOffset(nodes,nodes->file.latbins*end);
+
+    if(offset>index)
        lonbin=start;
     else
        lonbin=end;
    }
 
- while(lonbin<nodes->lonbins && nodes->offsets[lonbin*nodes->latbins]==nodes->offsets[(lonbin+1)*nodes->latbins])
+ while(lonbin<nodes->file.lonbins && 
+       LookupNodeOffset(nodes,lonbin*nodes->file.latbins)==LookupNodeOffset(nodes,(lonbin+1)*nodes->file.latbins))
     lonbin++;
 
  /* Search for latitude */
 
  start=0;
- end=nodes->latbins-1;
+ end=nodes->file.latbins-1;
 
  do
    {
-    mid=(start+end)/2;                                       /* Choose mid point */
+    mid=(start+end)/2;                  /* Choose mid point */
 
-    if(nodes->offsets[lonbin*nodes->latbins+mid]<index)      /* Mid point is too low */
+    offset=LookupNodeOffset(nodes,lonbin*nodes->file.latbins+mid);
+
+    if(offset<index)                    /* Mid point is too low */
        start=mid;
-    else if(nodes->offsets[lonbin*nodes->latbins+mid]>index) /* Mid point is too high */
+    else if(offset>index)               /* Mid point is too high */
        end=mid-1;
-    else                                                     /* Mid point is correct */
+    else                                /* Mid point is correct */
       {latbin=mid;break;}
    }
  while((end-start)>1);
 
  if(latbin==-1)
    {
-    if(nodes->offsets[lonbin*nodes->latbins+end]>index)
+    offset=LookupNodeOffset(nodes,lonbin*nodes->file.latbins+end);
+
+    if(offset>index)
        latbin=start;
     else
        latbin=end;
    }
 
- while(latbin<nodes->latbins && nodes->offsets[lonbin*nodes->latbins+latbin]==nodes->offsets[lonbin*nodes->latbins+latbin+1])
+ while(latbin<nodes->file.latbins &&
+       LookupNodeOffset(nodes,lonbin*nodes->file.latbins+latbin)==LookupNodeOffset(nodes,lonbin*nodes->file.latbins+latbin+1))
     latbin++;
 
  /* Return the values */
 
- *latitude =latlong_to_radians(bin_to_latlong(nodes->latzero+latbin)+off_to_latlong(node->latoffset));
- *longitude=latlong_to_radians(bin_to_latlong(nodes->lonzero+lonbin)+off_to_latlong(node->lonoffset));
+ *latitude =latlong_to_radians(bin_to_latlong(nodes->file.latzero+latbin)+off_to_latlong(node->latoffset));
+ *longitude=latlong_to_radians(bin_to_latlong(nodes->file.lonzero+lonbin)+off_to_latlong(node->lonoffset));
 }
