@@ -1,11 +1,9 @@
 /***************************************
- $Header: /home/amb/routino/src/RCS/osmparser.c,v 1.73 2010/11/13 14:22:28 amb Exp $
-
  OSM XML file parser (either JOSM or planet)
 
  Part of the Routino routing software.
  ******************/ /******************
- This file Copyright 2008-2010 Andrew M. Bishop
+ This file Copyright 2008-2011 Andrew M. Bishop
 
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU Affero General Public License as published by
@@ -43,6 +41,7 @@
 
 /* Macros */
 
+/*+ Checks if a value in the XML is one of the allowed values for true. +*/
 #define ISTRUE(xx) (!strcmp(xx,"true") || !strcmp(xx,"yes") || !strcmp(xx,"1"))
 
 
@@ -60,6 +59,9 @@ static way_t      *relation_ways=NULL;
 static int         relation_nways=0;
 static relation_t *relation_relations=NULL;
 static int         relation_nrelations=0;
+static way_t       relation_from=NO_WAY;
+static way_t       relation_to=NO_WAY;
+static node_t      relation_via=NO_NODE;
 
 static NodesX     *nodes;
 static SegmentsX  *segments;
@@ -340,6 +342,12 @@ static int memberType_function(const char *_tag_,int _type_,const char *type,con
           relation_nodes=(node_t*)realloc((void*)relation_nodes,(relation_nnodes+256)*sizeof(node_t));
 
        relation_nodes[relation_nnodes++]=node_id;
+
+       if(role)
+         {
+          if(!strcmp(role,"via"))
+             relation_via=node_id;
+         }
       }
     else if(!strcmp(type,"way"))
       {
@@ -349,6 +357,14 @@ static int memberType_function(const char *_tag_,int _type_,const char *type,con
           relation_ways=(way_t*)realloc((void*)relation_ways,(relation_nways+256)*sizeof(way_t));
 
        relation_ways[relation_nways++]=way_id;
+
+       if(role)
+         {
+          if(!strcmp(role,"from"))
+             relation_from=way_id;
+          if(!strcmp(role,"to"))
+             relation_to=way_id;
+         }
       }
     else if(!strcmp(type,"relation"))
       {
@@ -438,6 +454,10 @@ static int relationType_function(const char *_tag_,int _type_,const char *id)
 
     relation_nnodes=relation_nways=relation_nrelations=0;
 
+    relation_from=NO_WAY;
+    relation_to=NO_WAY;
+    relation_via=NO_NODE;
+
     /* Handle the relation information */
 
     XMLPARSE_ASSERT_STRING(_tag_,id); relation_id=atoll(id); /* need long long conversion */
@@ -554,8 +574,8 @@ int ParseOSM(FILE *file,NodesX *OSMNodes,SegmentsX *OSMSegments,WaysX *OSMWays,R
 
 static void process_node_tags(TagList *tags,node_t id,double latitude,double longitude)
 {
- allow_t allow=Allow_ALL;
-
+ transports_t allow=Transports_ALL;
+ uint16_t flags=0;
  int i;
 
  /* Parse the tags */
@@ -570,61 +590,65 @@ static void process_node_tags(TagList *tags,node_t id,double latitude,double lon
       case 'b':
        if(!strcmp(k,"bicycle"))
           if(!ISTRUE(v))
-             allow&=~Allow_Bicycle;
+             allow&=~Transports_Bicycle;
 
        break;
 
       case 'f':
        if(!strcmp(k,"foot"))
           if(!ISTRUE(v))
-             allow&=~Allow_Foot;
+             allow&=~Transports_Foot;
 
        break;
 
       case 'g':
        if(!strcmp(k,"goods"))
           if(!ISTRUE(v))
-             allow&=~Allow_Goods;
+             allow&=~Transports_Goods;
 
        break;
 
       case 'h':
+       if(!strcmp(k,"highway"))
+          if(!strcmp(v,"mini_roundabout"))
+             flags|=NODE_MINIRNDBT;
+
        if(!strcmp(k,"horse"))
           if(!ISTRUE(v))
-             allow&=~Allow_Horse;
+             allow&=~Transports_Horse;
 
        if(!strcmp(k,"hgv"))
           if(!ISTRUE(v))
-             allow&=~Allow_HGV;
+             allow&=~Transports_HGV;
 
        break;
 
       case 'm':
        if(!strcmp(k,"moped"))
           if(!ISTRUE(v))
-             allow&=~Allow_Moped;
+             allow&=~Transports_Moped;
 
        if(!strcmp(k,"motorbike"))
           if(!ISTRUE(v))
-             allow&=~Allow_Motorbike;
+             allow&=~Transports_Motorbike;
 
        if(!strcmp(k,"motorcar"))
           if(!ISTRUE(v))
-             allow&=~Allow_Motorcar;
+             allow&=~Transports_Motorcar;
 
        break;
 
       case 'p':
        if(!strcmp(k,"psv"))
           if(!ISTRUE(v))
-             allow&=~Allow_PSV;
+             allow&=~Transports_PSV;
 
        break;
 
       case 'w':
        if(!strcmp(k,"wheelchair"))
           if(!ISTRUE(v))
-             allow&=~Allow_Wheelchair;
+             allow&=~Transports_Wheelchair;
 
        break;
 
@@ -635,7 +659,7 @@ static void process_node_tags(TagList *tags,node_t id,double latitude,double lon
 
  /* Create the node */
 
- AppendNode(nodes,id,degrees_to_radians(latitude),degrees_to_radians(longitude),allow);
+ AppendNode(nodes,id,degrees_to_radians(latitude),degrees_to_radians(longitude),allow,flags);
 }
 
 
@@ -650,9 +674,8 @@ static void process_node_tags(TagList *tags,node_t id,double latitude,double lon
 static void process_way_tags(TagList *tags,way_t id)
 {
  Way   way={0};
- int   oneway=0,roundabout=0;
+ int   oneway=0;
  char *name=NULL,*ref=NULL;
-
  int i;
 
  /* Parse the tags */
@@ -667,7 +690,7 @@ static void process_way_tags(TagList *tags,way_t id)
       case 'b':
        if(!strcmp(k,"bicycle"))
           if(ISTRUE(v))
-             way.allow|= Allow_Bicycle;
+             way.allow|= Transports_Bicycle;
 
        if(!strcmp(k,"bicycleroute"))
           if(ISTRUE(v))
@@ -682,7 +705,7 @@ static void process_way_tags(TagList *tags,way_t id)
       case 'f':
        if(!strcmp(k,"foot"))
           if(ISTRUE(v))
-             way.allow|= Allow_Foot;
+             way.allow|= Transports_Foot;
 
        if(!strcmp(k,"footroute"))
           if(ISTRUE(v))
@@ -693,7 +716,7 @@ static void process_way_tags(TagList *tags,way_t id)
       case 'g':
        if(!strcmp(k,"goods"))
           if(ISTRUE(v))
-             way.allow|=Allow_Goods;
+             way.allow|=Transports_Goods;
 
        break;
 
@@ -703,17 +726,11 @@ static void process_way_tags(TagList *tags,way_t id)
 
        if(!strcmp(k,"horse"))
           if(ISTRUE(v))
-             way.allow|=Allow_Horse;
+             way.allow|=Transports_Horse;
 
        if(!strcmp(k,"hgv"))
           if(ISTRUE(v))
-             way.allow|=Allow_HGV;
-
-       break;
-
-      case 'j':
-       if(!strcmp(k,"junction") && !strcmp(v,"roundabout"))
-          roundabout=1;
+             way.allow|=Transports_HGV;
 
        break;
 
@@ -787,15 +804,15 @@ static void process_way_tags(TagList *tags,way_t id)
 
        if(!strcmp(k,"moped"))
           if(ISTRUE(v))
-             way.allow|=Allow_Moped;
+             way.allow|=Transports_Moped;
 
        if(!strcmp(k,"motorbike"))
           if(ISTRUE(v))
-             way.allow|=Allow_Motorbike;
+             way.allow|=Transports_Motorbike;
 
        if(!strcmp(k,"motorcar"))
           if(ISTRUE(v))
-             way.allow|=Allow_Motorcar;
+             way.allow|=Transports_Motorcar;
 
        if(!strcmp(k,"multilane"))
           if(ISTRUE(v))
@@ -827,7 +844,7 @@ static void process_way_tags(TagList *tags,way_t id)
 
        if(!strcmp(k,"psv"))
           if(ISTRUE(v))
-             way.allow|=Allow_PSV;
+             way.allow|=Transports_PSV;
 
        break;
 
@@ -847,7 +864,7 @@ static void process_way_tags(TagList *tags,way_t id)
       case 'w':
        if(!strcmp(k,"wheelchair"))
           if(ISTRUE(v))
-             way.allow|=Allow_Wheelchair;
+             way.allow|=Transports_Wheelchair;
 
        break;
 
@@ -867,9 +884,6 @@ static void process_way_tags(TagList *tags,way_t id)
        if(oneway)
           way.type|=Way_OneWay;
 
-       if(roundabout)
-          way.type|=Way_Roundabout;
-
        if(ref && name)
          {
           refname=(char*)malloc(strlen(ref)+strlen(name)+4);
@@ -893,20 +907,11 @@ static void process_way_tags(TagList *tags,way_t id)
           node_t to  =way_nodes[i];
 
           if(oneway>0)
-            {
              AppendSegment(segments,id,from,to,ONEWAY_1TO2);
-             AppendSegment(segments,id,to,from,ONEWAY_2TO1);
-            }
           else if(oneway<0)
-            {
              AppendSegment(segments,id,from,to,ONEWAY_2TO1);
-             AppendSegment(segments,id,to,from,ONEWAY_1TO2);
-            }
           else
-            {
              AppendSegment(segments,id,from,to,0);
-             AppendSegment(segments,id,to,from,0);
-            }
          }
       }
    }
@@ -923,7 +928,10 @@ static void process_way_tags(TagList *tags,way_t id)
 
 static void process_relation_tags(TagList *tags,relation_t id)
 {
- allow_t routes=Allow_None;
+ transports_t routes=Transports_None;
+ transports_t except=Transports_None;
+ int relation_turn_restriction=0;
+ TurnRestriction restriction=TurnRestrict_None;
  int i;
 
  /* Parse the tags */
@@ -938,17 +946,45 @@ static void process_relation_tags(TagList *tags,relation_t id)
       case 'b':
        if(!strcmp(k,"bicycleroute"))
           if(ISTRUE(v))
-             routes|=Allow_Bicycle;
+             routes|=Transports_Bicycle;
+
+       break;
+
+      case 'e':
+       if(!strcmp(k,"except"))
+          for(i=1;i<Transport_Count;i++)
+             if(!strstr(v,TransportName(i)))
+                except|=TRANSPORTS(i);
 
        break;
 
       case 'f':
        if(!strcmp(k,"footroute"))
           if(ISTRUE(v))
-             routes|=Allow_Foot;
+             routes|=Transports_Foot;
 
        break;
 
+      case 'r':
+       if(!strcmp(k,"restriction"))
+         {
+          if(!strcmp(v,"no_right_turn"   )) restriction=TurnRestrict_no_right_turn;
+          if(!strcmp(v,"no_left_turn"    )) restriction=TurnRestrict_no_left_turn;
+          if(!strcmp(v,"no_u_turn"       )) restriction=TurnRestrict_no_u_turn;
+          if(!strcmp(v,"no_straight_on"  )) restriction=TurnRestrict_no_straight_on;
+          if(!strcmp(v,"only_right_turn" )) restriction=TurnRestrict_only_right_turn;
+          if(!strcmp(v,"only_left_turn"  )) restriction=TurnRestrict_only_left_turn;
+          if(!strcmp(v,"only_straight_on")) restriction=TurnRestrict_only_straight_on;
+         }
+
+       break;
+
+      case 't':
+       if(!strcmp(k,"type"))
+          if(!strcmp(v,"restriction"))
+             relation_turn_restriction=1;
+       break;
+
       default:
        ;
       }
@@ -958,8 +994,16 @@ static void process_relation_tags(TagList *tags,relation_t id)
     relations even if they are not routes because they might be referenced by
     other relations that are routes) */
 
- if(relation_nways || relation_nrelations)
+ if((relation_nways || relation_nrelations) && !relation_turn_restriction)
     AppendRouteRelation(relations,id,routes,
                         relation_ways,relation_nways,
                         relation_relations,relation_nrelations);
+
+ /* Create the turn restriction relation. */
+
+ if(relation_turn_restriction && restriction!=TurnRestrict_None &&
+    relation_from!=NO_WAY && relation_to!=NO_WAY && relation_via!=NO_NODE)
+    AppendTurnRestrictRelation(relations,id,
+                               relation_from,relation_to,relation_via,
+                               restriction,except);
 }
