@@ -1,11 +1,9 @@
 /***************************************
- $Header: /home/amb/routino/src/RCS/profiles.c,v 1.42 2010/05/29 10:37:12 amb Exp $
-
  Load the profiles from a file and the functions for handling them.
 
  Part of the Routino routing software.
  ******************/ /******************
- This file Copyright 2008-2010 Andrew M. Bishop
+ This file Copyright 2008-2011 Andrew M. Bishop
 
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU Affero General Public License as published by
@@ -25,13 +23,18 @@
 #include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
+#include <math.h>
 
-#include "profiles.h"
 #include "types.h"
 #include "ways.h"
-#include "xmlparse.h"
-#include "functions.h"
 
+#include "files.h"
+#include "profiles.h"
+#include "functions.h"
+#include "xmlparse.h"
+
+
+/* Local variables */
 
 /*+ The profiles that have been loaded from file. +*/
 static Profile **loaded_profiles=NULL;
@@ -50,6 +53,7 @@ static int lengthType_function(const char *_tag_,int _type_,const char *limit);
 static int widthType_function(const char *_tag_,int _type_,const char *limit);
 static int heightType_function(const char *_tag_,int _type_,const char *limit);
 static int weightType_function(const char *_tag_,int _type_,const char *limit);
+static int turnsType_function(const char *_tag_,int _type_,const char *obey);
 //static int propertiesType_function(const char *_tag_,int _type_);
 static int onewayType_function(const char *_tag_,int _type_,const char *obey);
 static int propertyType_function(const char *_tag_,int _type_,const char *type,const char *percent);
@@ -110,6 +114,13 @@ static xmltag propertiesType_tag=
                NULL,
                {&propertyType_tag,NULL}};
 
+/*+ The turnsType type tag. +*/
+static xmltag turnsType_tag=
+              {"turns",
+               1, {"obey"},
+               turnsType_function,
+               {NULL}};
+
 /*+ The weightType type tag. +*/
 static xmltag weightType_tag=
               {"weight",
@@ -143,7 +154,7 @@ static xmltag restrictionsType_tag=
               {"restrictions",
                0, {NULL},
                NULL,
-               {&onewayType_tag,&weightType_tag,&heightType_tag,&widthType_tag,&lengthType_tag,NULL}};
+               {&onewayType_tag,&turnsType_tag,&weightType_tag,&heightType_tag,&widthType_tag,&lengthType_tag,NULL}};
 
 /*+ The profileType type tag. +*/
 static xmltag profileType_tag=
@@ -360,6 +371,33 @@ static int onewayType_function(const char *_tag_,int _type_,const char *obey)
 //}
 
 
+/*++++++++++++++++++++++++++++++++++++++
+  The function that is called when the turnsType XSD type is seen
+
+  int turnsType_function Returns 0 if no error occured or something else otherwise.
+
+  const char *_tag_ Set to the name of the element tag that triggered this function call.
+
+  int _type_ Set to XMLPARSE_TAG_START at the start of a tag and/or XMLPARSE_TAG_END at the end of a tag.
+
+  const char *obey The contents of the 'obey' attribute (or NULL if not defined).
+  ++++++++++++++++++++++++++++++++++++++*/
+
+static int turnsType_function(const char *_tag_,int _type_,const char *obey)
+{
+ if(_type_&XMLPARSE_TAG_START)
+   {
+    int o;
+
+    XMLPARSE_ASSERT_INTEGER(_tag_,obey,o);
+
+    loaded_profiles[nloaded_profiles-1]->turns=!!o;
+   }
+
+ return(0);
+}
+
+
 /*++++++++++++++++++++++++++++++++++++++
   The function that is called when the weightType XSD type is seen
 
@@ -616,7 +654,7 @@ int ParseXMLProfiles(const char *filename)
 
 
 /*++++++++++++++++++++++++++++++++++++++
-  Get the profile for a type of transport.
+  Get a named profile.
 
   Profile *GetProfile Returns a pointer to the profile.
 
@@ -636,7 +674,7 @@ Profile *GetProfile(const char *name)
 
 
 /*++++++++++++++++++++++++++++++++++++++
-  Update a profile with highway preference scaling factor.
+  Update a profile with the highway preference scaling factors.
 
   int UpdateProfile Returns 1 in case of a problem.
 
@@ -652,12 +690,12 @@ int UpdateProfile(Profile *profile,Ways *ways)
 
  /* Fix up the allowed transport types. */
 
- profile->allow=ALLOWED(profile->transport);
+ profile->allow=TRANSPORTS(profile->transport);
 
- if(!(profile->allow & ways->allow))
+ if(!(profile->allow & ways->file.allow))
     return(1);
 
- /* Normalise the highway preferences into the range 0 -> 1 */
+ /* Normalise the highway preferences into the range ~0 -> 1 */
 
  for(i=1;i<Way_Count;i++)
    {
@@ -672,9 +710,14 @@ int UpdateProfile(Profile *profile,Ways *ways)
     return(1);
 
  for(i=1;i<Way_Count;i++)
+   {
     profile->highway[i]/=hmax;
 
- /* Normalise the property preferences into the range 0 -> 2 */
+    if(profile->highway[i]<0.0001)
+       profile->highway[i]=0.0001;
+   }
+
+ /* Normalise the property preferences into the range ~0 -> 1 */
 
  for(i=1;i<Property_Count;i++)
    {
@@ -684,8 +727,21 @@ int UpdateProfile(Profile *profile,Ways *ways)
     if(profile->props_yes[i]>100)
        profile->props_yes[i]=100;
 
-    profile->props_yes[i]/=50;
-    profile->props_no [i] =2-profile->props_yes[i];
+    profile->props_yes[i]/=100;
+    profile->props_no [i] =1-profile->props_yes[i];
+
+    /* Squash the properties; selecting 60% preference without the sqrt() allows
+       routes 50% longer on highways with the property compared to ones without.
+       With the sqrt() function the ratio is only 22% allowing finer control. */
+
+    profile->props_yes[i] =sqrt(profile->props_yes[i]);
+    profile->props_no [i] =sqrt(profile->props_no[i] );
+
+    if(profile->props_yes[i]<0.0001)
+       profile->props_yes[i]=0.0001;
+
+    if(profile->props_no[i]<0.0001)
+       profile->props_no[i]=0.0001;
    }
 
  /* Find the fastest preferred speed */
@@ -704,7 +760,7 @@ int UpdateProfile(Profile *profile,Ways *ways)
  profile->max_pref=1; /* since highway prefs were normalised to 1 */
 
  for(i=1;i<Property_Count;i++)
-    if(ways->props & PROPERTIES(i))
+    if(ways->file.props & PROPERTIES(i))
       {
        if(profile->props_yes[i]>profile->props_no[i])
           profile->max_pref*=profile->props_yes[i];
@@ -751,6 +807,7 @@ void PrintProfile(const Profile *profile)
  printf("\n");
 
  printf("Obey one-way  : %s\n",profile->oneway?"yes":"no");
+ printf("Obey turns    : %s\n",profile->turns?"yes":"no");
  printf("Minimum weight: %.1f tonnes\n",weight_to_tonnes(profile->weight));
  printf("Minimum height: %.1f metres\n",height_to_metres(profile->height));
  printf("Minimum width : %.1f metres\n",width_to_metres(profile->width));
@@ -759,7 +816,7 @@ void PrintProfile(const Profile *profile)
 
 
 /*++++++++++++++++++++++++++++++++++++++
-  Print out the profiles as XML for use as program input.
+  Print out all of the loaded profiles as XML for use as program input.
   ++++++++++++++++++++++++++++++++++++++*/
 
 void PrintProfilesXML(void)
@@ -794,6 +851,7 @@ void PrintProfilesXML(void)
 
     printf("    <restrictions>\n");
     printf("      <oneway obey=\"%d\" /> \n",loaded_profiles[j]->oneway);
+    printf("      <turns  obey=\"%d\" /> \n",loaded_profiles[j]->turns);
     printf("      <weight limit=\"%.1f\" />\n",weight_to_tonnes(loaded_profiles[j]->weight));
     printf("      <height limit=\"%.1f\" />\n",height_to_metres(loaded_profiles[j]->height));
     printf("      <width  limit=\"%.1f\" />\n",width_to_metres(loaded_profiles[j]->width));
@@ -809,7 +867,7 @@ void PrintProfilesXML(void)
 
 
 /*++++++++++++++++++++++++++++++++++++++
-  Print out the profiles as JavaScript Object Notation for use in a web form.
+  Print out all of the loaded profiles as JavaScript Object Notation for use in a web page.
   ++++++++++++++++++++++++++++++++++++++*/
 
 void PrintProfilesJSON(void)
@@ -824,38 +882,38 @@ void PrintProfilesJSON(void)
  printf("\n");
 
  printf("  // Transport types\n");
- printf("  transports: {");
+ printf("  transports: { ");
  for(j=0;j<nloaded_profiles;j++)
-    printf("%s%s: %d",j==1?"":", ",TransportName(loaded_profiles[j]->transport),j);
- printf("},\n");
+    printf("%s%s: %d",j==0?"":", ",TransportName(loaded_profiles[j]->transport),j+1);
+ printf(" },\n");
  printf("\n");
 
  printf("  // Highway types\n");
- printf("  highways: {");
+ printf("  highways: { ");
  for(i=1;i<Way_Count;i++)
     printf("%s%s: %d",i==1?"":", ",HighwayName(i),i);
- printf("},\n");
+ printf(" },\n");
  printf("\n");
 
  printf("  // Property types\n");
- printf("  properties: {");
+ printf("  properties: { ");
  for(i=1;i<Property_Count;i++)
     printf("%s%s: %d",i==1?"":", ",PropertyName(i),i);
- printf("},\n");
+ printf(" },\n");
  printf("\n");
 
  printf("  // Restriction types\n");
- printf("  restrictions: {oneway: 1, weight: 2, height: 3, width: 4, length: 5},\n");
+ printf("  restrictions: { oneway: 1, turns: 2, weight: 3, height: 4, width: 5, length: 6 },\n");
  printf("\n");
 
  printf("  // Allowed highways\n");
  printf("  profile_highway: {\n");
  for(i=1;i<Way_Count;i++)
    {
-    printf("    %12s: {",HighwayName(i));
+    printf("    %12s: { ",HighwayName(i));
     for(j=0;j<nloaded_profiles;j++)
-       printf("%s%s: %3d",j==1?"":", ",TransportName(loaded_profiles[j]->transport),(int)loaded_profiles[j]->highway[i]);
-    printf("}%s\n",i==(Way_Count-1)?"":",");
+       printf("%s%s: %3d",j==0?"":", ",TransportName(loaded_profiles[j]->transport),(int)loaded_profiles[j]->highway[i]);
+    printf(" }%s\n",i==(Way_Count-1)?"":",");
    }
  printf("     },\n");
  printf("\n");
@@ -864,10 +922,10 @@ void PrintProfilesJSON(void)
  printf("  profile_speed: {\n");
  for(i=1;i<Way_Count;i++)
    {
-    printf("    %12s: {",HighwayName(i));
+    printf("    %12s: { ",HighwayName(i));
     for(j=0;j<nloaded_profiles;j++)
-       printf("%s%s: %3d",j==1?"":", ",TransportName(loaded_profiles[j]->transport),loaded_profiles[j]->speed[i]);
-    printf("}%s\n",i==(Way_Count-1)?"":",");
+       printf("%s%s: %3d",j==0?"":", ",TransportName(loaded_profiles[j]->transport),loaded_profiles[j]->speed[i]);
+    printf(" }%s\n",i==(Way_Count-1)?"":",");
    }
  printf("     },\n");
  printf("\n");
@@ -876,36 +934,40 @@ void PrintProfilesJSON(void)
  printf("  profile_property: {\n");
  for(i=1;i<Property_Count;i++)
    {
-    printf("    %12s: {",PropertyName(i));
+    printf("    %12s: { ",PropertyName(i));
     for(j=0;j<nloaded_profiles;j++)
-       printf("%s%s: %3d",j==1?"":", ",TransportName(loaded_profiles[j]->transport),(int)loaded_profiles[j]->props_yes[i]);
-    printf("}%s\n",i==(Property_Count-1)?"":",");
+       printf("%s%s: %3d",j==0?"":", ",TransportName(loaded_profiles[j]->transport),(int)loaded_profiles[j]->props_yes[i]);
+    printf(" }%s\n",i==(Property_Count-1)?"":",");
    }
  printf("     },\n");
  printf("\n");
 
  printf("  // Restrictions\n");
  printf("  profile_restrictions: {\n");
- printf("    %12s: {","oneway");
+ printf("    %12s: { ","oneway");
  for(j=0;j<nloaded_profiles;j++)
-    printf("%s%s: %4d",j==1?"":", ",TransportName(loaded_profiles[j]->transport),loaded_profiles[j]->oneway);
- printf("},\n");
- printf("    %12s: {","weight");
+    printf("%s%s: %4d",j==0?"":", ",TransportName(loaded_profiles[j]->transport),loaded_profiles[j]->oneway);
+ printf(" },\n");
+ printf("    %12s: { ","turns");
  for(j=0;j<nloaded_profiles;j++)
-    printf("%s%s: %4.1f",j==1?"":", ",TransportName(loaded_profiles[j]->transport),weight_to_tonnes(loaded_profiles[j]->weight));
- printf("},\n");
- printf("    %12s: {","height");
+    printf("%s%s: %4d",j==0?"":", ",TransportName(loaded_profiles[j]->transport),loaded_profiles[j]->turns);
+ printf(" },\n");
+ printf("    %12s: { ","weight");
  for(j=0;j<nloaded_profiles;j++)
-    printf("%s%s: %4.1f",j==1?"":", ",TransportName(loaded_profiles[j]->transport),height_to_metres(loaded_profiles[j]->height));
- printf("},\n");
- printf("    %12s: {","width");
+    printf("%s%s: %4.1f",j==0?"":", ",TransportName(loaded_profiles[j]->transport),weight_to_tonnes(loaded_profiles[j]->weight));
+ printf(" },\n");
+ printf("    %12s: { ","height");
  for(j=0;j<nloaded_profiles;j++)
-    printf("%s%s: %4.1f",j==1?"":", ",TransportName(loaded_profiles[j]->transport),width_to_metres(loaded_profiles[j]->width));
- printf("},\n");
- printf("    %12s: {","length");
+    printf("%s%s: %4.1f",j==0?"":", ",TransportName(loaded_profiles[j]->transport),height_to_metres(loaded_profiles[j]->height));
+ printf(" },\n");
+ printf("    %12s: { ","width");
  for(j=0;j<nloaded_profiles;j++)
-    printf("%s%s: %4.1f",j==1?"":", ",TransportName(loaded_profiles[j]->transport),length_to_metres(loaded_profiles[j]->length));
- printf("}\n");
+    printf("%s%s: %4.1f",j==0?"":", ",TransportName(loaded_profiles[j]->transport),width_to_metres(loaded_profiles[j]->width));
+ printf(" },\n");
+ printf("    %12s: { ","length");
+ for(j=0;j<nloaded_profiles;j++)
+    printf("%s%s: %4.1f",j==0?"":", ",TransportName(loaded_profiles[j]->transport),length_to_metres(loaded_profiles[j]->length));
+ printf(" }\n");
  printf("     }\n");
  printf("\n");
 
@@ -914,7 +976,7 @@ void PrintProfilesJSON(void)
 
 
 /*++++++++++++++++++++++++++++++++++++++
-  Print out the profiles as Perl for use in a web CGI.
+  Print out all of the loaded profiles as Perl for use in a web CGI.
   ++++++++++++++++++++++++++++++++++++++*/
 
 void PrintProfilesPerl(void)
@@ -929,28 +991,28 @@ void PrintProfilesPerl(void)
  printf("\n");
 
  printf("  # Transport types\n");
- printf("  transports => {");
+ printf("  transports => { ");
  for(j=0;j<nloaded_profiles;j++)
-    printf("%s%s => %d",j==1?"":", ",TransportName(loaded_profiles[j]->transport),j);
- printf("},\n");
+    printf("%s%s => %d",j==0?"":", ",TransportName(loaded_profiles[j]->transport),j+1);
+ printf(" },\n");
  printf("\n");
 
  printf("  # Highway types\n");
- printf("  highways => {");
+ printf("  highways => { ");
  for(i=1;i<Way_Count;i++)
     printf("%s%s => %d",i==1?"":", ",HighwayName(i),i);
- printf("},\n");
+ printf(" },\n");
  printf("\n");
 
  printf("  # Property types\n");
- printf("  properties => {");
+ printf("  properties => { ");
  for(i=1;i<Property_Count;i++)
     printf("%s%s => %d",i==1?"":", ",PropertyName(i),i);
- printf("},\n");
+ printf(" },\n");
  printf("\n");
 
  printf("  # Restriction types\n");
- printf("  restrictions => {oneway => 1, weight => 2, height => 3, width => 4, length => 5},\n");
+ printf("  restrictions => { oneway => 1, turns => 2, weight => 3, height => 4, width => 5, length => 6 },\n");
  printf("\n");
 
  printf("  # Allowed highways\n");
@@ -959,8 +1021,8 @@ void PrintProfilesPerl(void)
    {
     printf("  %12s => {",HighwayName(i));
     for(j=0;j<nloaded_profiles;j++)
-       printf("%s %s => %3d",j==1?"":", ",TransportName(loaded_profiles[j]->transport),(int)loaded_profiles[j]->highway[i]);
-    printf("}%s\n",i==(Way_Count-1)?"":",");
+       printf("%s %s => %3d",j==0?"":", ",TransportName(loaded_profiles[j]->transport),(int)loaded_profiles[j]->highway[i]);
+    printf(" }%s\n",i==(Way_Count-1)?"":",");
    }
  printf("     },\n");
  printf("\n");
@@ -971,8 +1033,8 @@ void PrintProfilesPerl(void)
    {
     printf("  %12s => {",HighwayName(i));
     for(j=0;j<nloaded_profiles;j++)
-       printf("%s %s => %3d",j==1?"":", ",TransportName(loaded_profiles[j]->transport),loaded_profiles[j]->speed[i]);
-    printf("}%s\n",i==(Way_Count-1)?"":",");
+       printf("%s %s => %3d",j==0?"":", ",TransportName(loaded_profiles[j]->transport),loaded_profiles[j]->speed[i]);
+    printf(" }%s\n",i==(Way_Count-1)?"":",");
    }
  printf("     },\n");
  printf("\n");
@@ -983,8 +1045,8 @@ void PrintProfilesPerl(void)
    {
     printf("  %12s => {",PropertyName(i));
     for(j=0;j<nloaded_profiles;j++)
-       printf("%s %s => %3d",j==1?"":", ",TransportName(loaded_profiles[j]->transport),(int)loaded_profiles[j]->props_yes[i]);
-    printf("}%s\n",i==(Property_Count-1)?"":",");
+       printf("%s %s => %3d",j==0?"":", ",TransportName(loaded_profiles[j]->transport),(int)loaded_profiles[j]->props_yes[i]);
+    printf(" }%s\n",i==(Property_Count-1)?"":",");
    }
  printf("     },\n");
  printf("\n");
@@ -993,25 +1055,29 @@ void PrintProfilesPerl(void)
  printf("  profile_restrictions => {\n");
  printf("    %12s => {","oneway");
  for(j=0;j<nloaded_profiles;j++)
-    printf("%s %s => %4d",j==1?"":", ",TransportName(loaded_profiles[j]->transport),loaded_profiles[j]->oneway);
- printf("},\n");
+    printf("%s %s => %4d",j==0?"":", ",TransportName(loaded_profiles[j]->transport),loaded_profiles[j]->oneway);
+ printf(" },\n");
+ printf("    %12s => {","turns");
+ for(j=0;j<nloaded_profiles;j++)
+    printf("%s %s => %4d",j==0?"":", ",TransportName(loaded_profiles[j]->transport),loaded_profiles[j]->turns);
+ printf(" },\n");
  printf("    %12s => {","weight");
  for(j=0;j<nloaded_profiles;j++)
-    printf("%s %s => %4.1f",j==1?"":", ",TransportName(loaded_profiles[j]->transport),weight_to_tonnes(loaded_profiles[j]->weight));
- printf("},\n");
+    printf("%s %s => %4.1f",j==0?"":", ",TransportName(loaded_profiles[j]->transport),weight_to_tonnes(loaded_profiles[j]->weight));
+ printf(" },\n");
  printf("    %12s => {","height");
  for(j=0;j<nloaded_profiles;j++)
-    printf("%s %s => %4.1f",j==1?"":", ",TransportName(loaded_profiles[j]->transport),height_to_metres(loaded_profiles[j]->height));
- printf("},\n");
+    printf("%s %s => %4.1f",j==0?"":", ",TransportName(loaded_profiles[j]->transport),height_to_metres(loaded_profiles[j]->height));
+ printf(" },\n");
  printf("    %12s => {","width");
  for(j=0;j<nloaded_profiles;j++)
-    printf("%s %s => %4.1f",j==1?"":", ",TransportName(loaded_profiles[j]->transport),width_to_metres(loaded_profiles[j]->width));
- printf("},\n");
+    printf("%s %s => %4.1f",j==0?"":", ",TransportName(loaded_profiles[j]->transport),width_to_metres(loaded_profiles[j]->width));
+ printf(" },\n");
  printf("    %12s => {","length");
  for(j=0;j<nloaded_profiles;j++)
-    printf("%s %s => %4.1f",j==1?"":", ",TransportName(loaded_profiles[j]->transport),length_to_metres(loaded_profiles[j]->length));
- printf("}\n");
- printf("     },\n");
+    printf("%s %s => %4.1f",j==0?"":", ",TransportName(loaded_profiles[j]->transport),length_to_metres(loaded_profiles[j]->length));
+ printf(" }\n");
+ printf("     }\n");
  printf("\n");
 
  printf("}; # end of routino variable\n");
